#include <hypro/representations/GeometricObjectBase.h>
#include <hypro/util/plotting/Plotter.h>
#include <iostream>

using namespace hypro;

int main(int argc, char **argv) {
  using Number = double;

  // plotting
  Plotter<Number> &plotter = Plotter<Number>::getInstance();
  plotting::gnuplotSettings settings = plotter.settings();
  settings.keepAspectRatio = false;
  settings.linewidth = 1.5;
  plotter.updateSettings(settings);
  plotter.setFilename("sf");
  std::vector<Point<Number>> points;

  // create array holding equaly distributed directions
  int resolution = 180;
  if (argc == 2) {
    resolution = std::stoi(argv[1]);
  }

  matrix_t<Number> evaldirections = matrix_t<Number>(resolution, 2);
  for (int pos = 0; pos < resolution; ++pos) {
    double angle = pos * (360 / resolution);
    evaldirections(pos, 0) =
        carl::rationalize<Number>(cos(angle * 3.141592654 / 180));
    evaldirections(pos, 1) =
        carl::rationalize<Number>(sin(angle * 3.141592654 / 180));
  }

  // set up some constraints (for a box), defining the set of points x for which
  // Ax <= b (A = "matrix", b = "distances")
  matrix_t<Number> matrix = matrix_t<Number>(4, 2);
  vector_t<Number> distances = vector_t<Number>(4);
  matrix << 0, 1, 1, 0, 0, -1, -1, 0;
  distances << 1, 5, 1, -4;
  // defines the box: [4,5] x [-1,1]

  // support function containing all points x, where matrix^T*x <= distances
  // holds
  SupportFunction<Number> poly1(matrix, distances);

  // support function representing a 2-dimensional ball (euclidean-norm
  // [2-norm]) of radius 0.5
  SupportFunction<Number> ball(SF_TYPE::TWO_BALL, 0.5, 2);

  // support function which is the Minkowski-sum of the ball and the polytope.
  SupportFunction<Number> minkowski = poly1.minkowskiSum(ball);

  // write dot file of support function
  std::fstream out;
  out.open("sfTree.dot", std::fstream::out);
  out << minkowski.getDotRepresentation();
  out << "\n";
  out.close();

  // the vector holds all results (distances) for the evaluation (query) of the
  // support function.
  std::vector<EvaluationResult<Number>> ballEval =
      ball.multiEvaluate(evaldirections);
  std::vector<EvaluationResult<Number>> polyEval =
      poly1.multiEvaluate(evaldirections);
  std::vector<EvaluationResult<Number>> minkowskiEval =
      minkowski.multiEvaluate(evaldirections);

  // we create a 2-dimensional h-polytope from our evaluation results
  {
    HPolytope<Number> tmp;
    for (int i = 0; i < resolution; ++i) {
      tmp.insert(
          Halfspace<Number>(evaldirections.row(i), ballEval[i].supportValue));
    }
    unsigned original =
        plotter.addObject(tmp.vertices(), plotting::colors[plotting::green]);
  }

  {
    HPolytope<Number> tmp;
    for (int i = 0; i < resolution; ++i) {
      tmp.insert(
          Halfspace<Number>(evaldirections.row(i), polyEval[i].supportValue));
    }
    unsigned original =
        plotter.addObject(tmp.vertices(), plotting::colors[plotting::red]);
  }

  {
    HPolytope<Number> tmp;
    for (int i = 0; i < resolution; ++i) {
      tmp.insert(Halfspace<Number>(evaldirections.row(i),
                                   minkowskiEval[i].supportValue));
    }
    unsigned original =
        plotter.addObject(tmp.vertices(), plotting::colors[plotting::blue]);
  }

  // create actual gnuplot file
  plotter.plot2d(PLOTTYPE::png, true);

  return 0;
}
